// Generated by CoffeeScript 1.6.2
(function() {
  var Match, NoMatch, Parser, ParserState, alt, chain, check, commit, consume, debug, drop, end, implicit, not_, optional, parse, parser_state, reduce, regex, reject, repeat, repeatIgnore, repeatSeparated, resolve, seq, seqIgnore, string, util, _parser_id,
    __slice = [].slice;

  util = require('util');

  debug = require("./debugging").debug;

  parser_state = require("./parser_state");

  ParserState = parser_state.ParserState;

  Match = parser_state.Match;

  NoMatch = parser_state.NoMatch;

  _parser_id = 0;

  Parser = (function() {
    function Parser(_message, matcher) {
      this._message = _message;
      this.matcher = matcher;
      this.id = _parser_id++;
    }

    Parser.prototype.message = function() {
      var rv;

      if (this.recursing) {
        return "...";
      }
      this.recursing = true;
      rv = typeof this._message === "function" ? this._message() : this._message;
      this.recursing = false;
      return rv;
    };

    Parser.prototype.toString = function() {
      return "Parser(" + (this.message()) + ")";
    };

    Parser.prototype.fail = function(state, cont) {
      return cont(new NoMatch(state, "Expected " + this.message()));
    };

    Parser.prototype.parse = function(state, cont) {
      var entry, newCont, r, _i, _len, _ref, _results,
        _this = this;

      state = state.deeper();
      debug(function() {
        return ("-> state=" + state + "\n") + ("   parse (" + _this.id + "): " + _this);
      });
      newCont = function(rv) {
        debug(function() {
          return "<- " + rv;
        });
        return cont(rv);
      };
      entry = state.getCache(this);
      if (entry.continuations.length === 0) {
        entry.continuations.push(newCont);
        return state.addJob((function() {
          return "parse: " + state + ", " + (_this.toString());
        }), function() {
          return _this.matcher(state, function(rv) {
            var c, found, r, _i, _j, _len, _len1, _ref, _ref1, _results;

            found = false;
            _ref = entry.results;
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              r = _ref[_i];
              if (r.equals(rv)) {
                found = true;
              }
            }
            if (!found) {
              debug(function() {
                return ("<- coming back from " + _this + "\n") + ("   cache++ (" + _this.id + "," + state.pos + ") = " + rv);
              });
              entry.results.push(rv);
              _ref1 = entry.continuations;
              _results = [];
              for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
                c = _ref1[_j];
                _results.push(c(rv));
              }
              return _results;
            }
          });
        });
      } else {
        debug(function() {
          return "<- answer from cache (" + _this.id + "," + state.pos + "): " + (util.inspect(entry.results));
        });
        entry.continuations.push(newCont);
        _ref = entry.results;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          r = _ref[_i];
          _results.push(newCont(r));
        }
        return _results;
      }
    };

    Parser.prototype.onFail = function(newMessage) {
      var _this = this;

      return new Parser(this._message, function(state, cont) {
        return _this.parse(state, function(rv) {
          if (rv.ok) {
            return cont(rv);
          }
          return cont(new NoMatch(rv.state, newMessage, rv.abort));
        });
      });
    };

    Parser.prototype.onMatch = function(f) {
      var _this = this;

      return new Parser(this._message, function(state, cont) {
        return _this.parse(state, function(rv) {
          var e;

          if (!rv.ok) {
            return cont(rv);
          }
          rv = (function() {
            if (typeof f === "function") {
              try {
                return new Match(rv.state, f(rv.match), rv.commit);
              } catch (_error) {
                e = _error;
                return new NoMatch(state, e.toString());
              }
            } else {
              return new Match(rv.state, f, rv.commit);
            }
          })();
          return cont(rv);
        });
      });
    };

    Parser.prototype.matchIf = function(f) {
      var _this = this;

      return new Parser(this._message, function(state, cont) {
        return _this.parse(state, function(rv) {
          if (!rv.ok) {
            return cont(rv);
          }
          if (!f(rv.match)) {
            return _this.fail(state, cont);
          }
          return cont(rv);
        });
      });
    };

    Parser.prototype.then = function(p) {
      return chain(this, p, function(a, b) {
        return [a, b];
      });
    };

    Parser.prototype.or = function() {
      var others;

      others = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return alt.apply(null, [this].concat(__slice.call(others)));
    };

    Parser.prototype.optional = function(defaultValue) {
      if (defaultValue == null) {
        defaultValue = "";
      }
      return optional(this, defaultValue);
    };

    Parser.prototype.check = function() {
      return check(this);
    };

    Parser.prototype.commit = function() {
      return commit(this);
    };

    Parser.prototype.not_ = function() {
      return not_(this);
    };

    Parser.prototype.drop = function() {
      return drop(this);
    };

    Parser.prototype.repeat = function(minCount, maxCount) {
      return repeat(this, minCount, maxCount);
    };

    Parser.prototype.times = function(count) {
      return repeat(this, count, count);
    };

    return Parser;

  })();

  end = new Parser("end", function(state, cont) {
    if (state.pos === state.end) {
      return cont(new Match(state, null));
    } else {
      return this.fail(state, cont);
    }
  });

  reject = new Parser("failure", function(state, cont) {
    return this.fail(state, cont);
  });

  string = function(s) {
    var len;

    len = s.length;
    return new Parser("'" + s + "'", function(state, cont) {
      var candidate;

      candidate = state.text.slice(state.pos, state.pos + len);
      if (candidate === s) {
        return cont(new Match(state.advance(len), candidate));
      } else {
        return state.fail(cont, this.message());
      }
    });
  };

  regex = function(r) {
    var i, m, r2, source;

    i = r.ignoreCase ? "i" : "";
    m = r.multiline ? "m" : "";
    source = r.source[0] === "^" ? r.source : "^" + r.source;
    r2 = new RegExp(source, i + m);
    return new Parser(r.toString(), function(state, cont) {
      m = r2.exec(state.text.slice(state.pos));
      if (m != null) {
        return cont(new Match(state.advance(m[0].length), m));
      } else {
        return this.fail(state, cont);
      }
    });
  };

  optional = function(p, defaultValue) {
    if (defaultValue == null) {
      defaultValue = "";
    }
    p = implicit(p);
    return new Parser((function() {
      return "optional " + (resolve(p).message());
    }), function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (rv.ok) {
          return cont(rv);
        }
        return cont(new Match(state, defaultValue));
      });
    });
  };

  check = function(p) {
    p = implicit(p);
    return new Parser((function() {
      return resolve(p).message();
    }), function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (!rv.ok) {
          return cont(rv);
        }
        return cont(new Match(state, rv.match, rv.commit));
      });
    });
  };

  commit = function(p) {
    p = implicit(p);
    return new Parser((function() {
      return resolve(p).message();
    }), function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (!rv.ok) {
          return cont(rv);
        }
        rv.commit = true;
        debug(function() {
          return "commit!";
        });
        return cont(rv);
      });
    });
  };

  not_ = function(p) {
    var message;

    p = implicit(p);
    message = function() {
      return "not " + resolve(p).message();
    };
    return new Parser(message, function(state, cont) {
      p = resolve(p);
      return p.parse(state, function(rv) {
        if (rv.ok) {
          return state.fail(cont, message());
        } else {
          return cont(new Match(state, ""));
        }
      });
    });
  };

  drop = function(p) {
    return implicit(p).onMatch(function(x) {
      return null;
    });
  };

  chain = function(p1, p2, combiner) {
    return new Parser((function() {
      return "(" + (resolve(p1).message()) + ") chain (" + (resolve(p2).message()) + ")";
    }), function(state, cont) {
      p1 = resolve(p1);
      return p1.parse(state, function(rv1) {
        if (!rv1.ok) {
          return cont(rv1);
        }
        p2 = resolve(p2);
        return p2.parse(rv1.state, function(rv2) {
          if (!rv2.ok) {
            if (rv1.commit) {
              rv2.abort = true;
            }
            return cont(rv2);
          }
          return cont(new Match(rv2.state, combiner(rv1.match, rv2.match), rv2.commit));
        });
      });
    });
  };

  seq = function() {
    var combiner, message, p, parsers, rv, _i, _len;

    parsers = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    parsers = (function() {
      var _i, _len, _results;

      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    message = function() {
      return ((function() {
        var _i, _len, _results;

        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push("(" + resolve(p).message() + ")");
        }
        return _results;
      })()).join(" then ");
    };
    combiner = function(sum, x) {
      sum = sum.slice(0);
      if (x != null) {
        sum.push(x);
      }
      return sum;
    };
    rv = new Parser("''", function(state, cont) {
      return cont(new Match(state, []));
    });
    for (_i = 0, _len = parsers.length; _i < _len; _i++) {
      p = parsers[_i];
      rv = chain(rv, p, combiner);
    }
    return new Parser(message, function(state, cont) {
      return rv.parse(state, cont);
    });
  };

  seqIgnore = function() {
    var ignore, message, newseq, p, parsers, rv, _i, _len;

    ignore = arguments[0], parsers = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    parsers = (function() {
      var _i, _len, _results;

      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    message = function() {
      return ((function() {
        var _i, _len, _results;

        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push("(" + resolve(p).message() + ")");
        }
        return _results;
      })()).join(" then ");
    };
    newseq = [];
    for (_i = 0, _len = parsers.length; _i < _len; _i++) {
      p = parsers[_i];
      newseq.push(optional(ignore).drop());
      newseq.push(p);
    }
    rv = seq.apply(null, newseq);
    return new Parser(message, function(state, cont) {
      return rv.parse(state, cont);
    });
  };

  alt = function() {
    var message, p, parsers;

    parsers = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    parsers = (function() {
      var _i, _len, _results;

      _results = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        p = parsers[_i];
        _results.push(implicit(p));
      }
      return _results;
    })();
    message = function() {
      return ((function() {
        var _i, _len, _results;

        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push("(" + resolve(p).message() + ")");
        }
        return _results;
      })()).join(" or ");
    };
    return new Parser(message, function(state, cont) {
      var aborting, _i, _len, _ref, _results;

      parsers = (function() {
        var _i, _len, _results;

        _results = [];
        for (_i = 0, _len = parsers.length; _i < _len; _i++) {
          p = parsers[_i];
          _results.push(resolve(p));
        }
        return _results;
      })();
      debug(function() {
        return ("<alt> start: " + state + "\n") + ((function() {
          var _i, _len, _results;

          _results = [];
          for (_i = 0, _len = parsers.length; _i < _len; _i++) {
            p = parsers[_i];
            _results.push("<alt> -- " + p + "\n");
          }
          return _results;
        })()) + "<alt> --.";
      });
      aborting = false;
      _ref = parsers.slice(0);
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        p = _ref[_i];
        _results.push((function(p) {
          var _this = this;

          return state.addJob((function() {
            return "alt: " + state + ", " + (p.message());
          }), function() {
            debug(function() {
              return "<alt> next try: " + p + " at " + state;
            });
            if (aborting) {
              debug(function() {
                return "<alt> -- er, n/m, aborting";
              });
              return;
            }
            return p.parse(state, function(rv) {
              if (rv.abort) {
                aborting = true;
              }
              return cont(rv);
            });
          });
        })(p));
      }
      return _results;
    });
  };

  repeat = function(p, minCount, maxCount) {
    var countMessage, message;

    if (minCount == null) {
      minCount = 0;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    p = implicit(p);
    if (maxCount != null) {
      countMessage = "{" + minCount + ", " + maxCount + "}";
    } else {
      countMessage = "{" + minCount + "+}";
      maxCount = Math.pow(2, 31);
    }
    message = function() {
      return "(" + resolve(p).message() + (")" + countMessage);
    };
    return new Parser(message, function(state, cont) {
      var count, nextCont, origState;

      p = resolve(p);
      origState = state;
      count = 0;
      nextCont = function(rv, list, lastState) {
        var _this = this;

        if (list == null) {
          list = [];
        }
        if (lastState == null) {
          lastState = origState;
        }
        if (!rv.ok) {
          if (count >= minCount) {
            return cont(new Match(lastState, list, rv.commit));
          }
          return origState.fail(cont, message());
        }
        count += 1;
        if (rv.match != null) {
          list.push(rv.match);
        }
        if (count < maxCount) {
          if (rv.state.pos === origState.pos) {
            throw new Error("Repeating parser isn't making progress: " + rv.state.pos + "=" + origState.pos + " " + p);
          }
          return rv.state.addJob((function() {
            return "repeat: " + state + ", " + (message());
          }), function() {
            return p.parse(rv.state, function(x) {
              return nextCont(x, list.slice(0), rv.state);
            });
          });
        } else {
          return cont(new Match(rv.state, list, rv.commit));
        }
      };
      return p.parse(origState, nextCont);
    });
  };

  repeatIgnore = function(ignore, p, minCount, maxCount) {
    var p2;

    if (minCount == null) {
      minCount = 0;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    p2 = seq(optional(ignore).drop(), p).onMatch(function(x) {
      return x[0];
    });
    return repeat(p2, minCount, maxCount);
  };

  repeatSeparated = function(p, separator, minCount, maxCount) {
    var p2;

    if (separator == null) {
      separator = "";
    }
    if (minCount == null) {
      minCount = 1;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    p2 = seq(drop(separator), p).onMatch(function(x) {
      return x[0];
    });
    return seq(p, repeat(p2, minCount - 1, maxCount != null ? maxCount - 1 : maxCount)).onMatch(function(x) {
      return [x[0]].concat(x[1]);
    });
  };

  reduce = function(p, separator, accumulator, reducer, minCount, maxCount) {
    if (separator == null) {
      separator = "";
    }
    if (accumulator == null) {
      accumulator = null;
    }
    if (reducer == null) {
      reducer = null;
    }
    if (minCount == null) {
      minCount = 1;
    }
    if (maxCount == null) {
      maxCount = null;
    }
    if (accumulator == null) {
      accumulator = function(x) {
        return [x];
      };
    }
    if (reducer == null) {
      reducer = function(sum, sep, x) {
        return sum.push(x);
      };
    }
    return seq(p, repeat(seq(separator, p), minCount - 1, maxCount != null ? maxCount - 1 : maxCount)).onMatch(function(x) {
      return [accumulator(x[0])].concat(x[1]).reduce(function(sum, item) {
        return reducer(sum, item[0], item[1]);
      });
    });
  };

  implicit = function(p) {
    var className;

    if (typeof p === "string") {
      return string(p);
    }
    className = p.constructor.toString().split(" ")[1];
    if (className === "RegExp()") {
      return regex(p);
    }
    if (className === "Array()") {
      return seq.apply(null, p);
    }
    return p;
  };

  resolve = function(p) {
    if (!(typeof p === "function")) {
      return implicit(p);
    }
    p = implicit(p());
    if (p == null) {
      throw new Error("Can't resolve parser");
    }
    return p;
  };

  parse = function(p, str) {
    var failures, state, successes,
      _this = this;

    state = str instanceof ParserState ? str : new ParserState(str);
    p = resolve(p);
    successes = [];
    failures = [];
    state.addJob((function() {
      return "start: " + state + ", " + (p.toString());
    }), function() {
      return p.parse(state, function(rv) {
        if (rv.ok) {
          debug(function() {
            return "--- registering success: " + rv;
          });
          return successes.push(rv);
        } else {
          debug(function() {
            return "--- registering failure: " + rv;
          });
          return failures.push(rv);
        }
      });
    });
    while (state.trampoline.ready() && successes.length === 0) {
      state.trampoline.next();
    }
    debug(function() {
      var x;

      return "--- final tally:\n" + ((function() {
        var _i, _len, _results;

        _results = [];
        for (_i = 0, _len = successes.length; _i < _len; _i++) {
          x = successes[_i];
          _results.push("+++ " + x + "\n");
        }
        return _results;
      })()) + ((function() {
        var _i, _len, _results;

        _results = [];
        for (_i = 0, _len = failures.length; _i < _len; _i++) {
          x = failures[_i];
          _results.push("--- " + x + "\n");
        }
        return _results;
      })()) + "--- GOOD DAY SIR";
    });
    if (successes.length > 0) {
      return successes[0];
    } else {
      return failures[0];
    }
  };

  consume = function(p, str) {
    p = chain(implicit(p), end, function(a, b) {
      return a;
    });
    return parse(p, str);
  };

  exports.Parser = Parser;

  exports.end = end;

  exports.reject = reject;

  exports.string = string;

  exports.regex = regex;

  exports.optional = optional;

  exports.check = check;

  exports.commit = commit;

  exports.not_ = not_;

  exports.drop = drop;

  exports.seq = seq;

  exports.seqIgnore = seqIgnore;

  exports.alt = alt;

  exports.repeat = repeat;

  exports.repeatIgnore = repeatIgnore;

  exports.repeatSeparated = repeatSeparated;

  exports.reduce = reduce;

  exports.implicit = implicit;

  exports.parse = parse;

  exports.consume = consume;

}).call(this);
